"""
*****************************************************************
Licensed Materials - Property of IBM
(C) Copyright IBM Corp. 2019. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*****************************************************************
"""

import tensorflow as tf
import ddl
import numpy as np
import sys
from os import environ
import argparse


def variable_bcast_test (data, test_name):
    print ("Running test for " + test_name)

    X = tf.Variable(data, name="X")
    Y = tf.Variable(data, name="Y")

    # Initialize the variables (i.e. assign their default value)
    init = tf.global_variables_initializer()

    # Start session
    with tf.Session() as sess:
        #Run the initializer
        sess.run(init)

        print ("Before bcast op: DDL RANK: " + str(ddl_rank) + " X = " + str(sess.run(X)) + " Y = " + str(sess.run(Y)))

        Z = sess.run(X + Y)
        sess.run(ddl.get_global_variable_bcast_op())

        #Since the lowest rank is 0, we expect X and Y on all ranks to be 0
        x_val = sess.run(X)
        y_val = sess.run(Y)

        is_x_not_zero = np.all(x_val != 0)
        is_y_not_zero = np.all(y_val != 0)
        is_z_incorrect = np.all(Z != ddl.rank()*2)

        if ((is_x_not_zero) or (is_y_not_zero) or (is_z_incorrect)):
            print ("get_global_variable_bcast_op failed to send rank 0's " \
                    "variable values to other ranks")
            sys.exit(1)

        print ("After bcast op: DDL RANK: " + str(ddl_rank) + " X = " + str(x_val) + " Y = " + str(y_val) + " Z = " + str(Z))

if __name__ == '__main__':
    parser = argparse.ArgumentParser(
        description='DDL global variable bcast op test')
    parser.add_argument('--ddl_options', type=str,
                        default='-mode b:2x1 -dump_iter 100',
                        help='DDL options. e.g. "-mode b:2x1 -dump_iter 100"')
    FLAGS, unparsed = parser.parse_known_args()

    if environ.get('DDL_OPTIONS') is None:
        #Need to explicitly call ddl.init
        ddl.init(FLAGS.ddl_options)

    ddl_rank = ddl.rank()
    variable_bcast_test([ddl_rank], "Scalar int")

    float16_arr = np.ones((4,4), dtype=np.float16) * ddl_rank
    variable_bcast_test(float16_arr, "4x4 array of float16")

    float32_arr = np.ones((4,4), dtype=np.float32) * ddl_rank
    variable_bcast_test(float32_arr, "4x4 array of float32")
